@@ -1,7 +1,7 @@
 #  File src/library/base/R/load.R
-#  Part of the R package, http://www.R-project.org
+#  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2015 The R Core Team
+#  Copyright (C) 1995-2022 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 #  GNU General Public License for more details.
 #
 #  A copy of the GNU General Public License is available at
-#  http://www.r-project.org/Licenses/
+#  https://www.R-project.org/Licenses/
 
 load <- function (file, envir = parent.frame(), verbose = FALSE)
 {
@@ -27,11 +27,11 @@ load <- function (file, envir = parent.frame(), verbose = FALSE)
         ## and closes it again.
         magic <- readChar(con, 5L, useBytes = TRUE)
 	if (!length(magic)) stop("empty (zero-byte) input file")
-	if (!grepl("RD[AX]2\n", magic)) {
+	if (!grepl("RD[ABX][2-9]\n", magic)) {
             ## a check while we still know the call to load()
-            if(grepl("RD[ABX][12]\r", magic))
+            if(grepl("RD[ABX][2-9]\r", magic))
                 stop("input has been corrupted, with LF replaced by CR")
-            ## Not a version 2 magic number, so try the pre-R-1.4.0 code
+            ## Not a version 2 or higher magic number, so try the pre-R-1.4.0 code
             warning(sprintf("file %s has magic number '%s'\n",
                             sQuote(basename(file)),
                             gsub("[\n\r]*", "", magic)),
@@ -171,6 +171,9 @@ sys.load.image <- function(name, quiet)
 {
     if (file.exists(name)) {
         load(name, envir = .GlobalEnv)
+        sample.kind <- .Internal(RNGkind(NULL, NULL, NULL))[3L]
+        if (sample.kind == 0L)
+            warning("non-uniform 'Rounding' sampler used", domain = NA)
         if (! quiet)
 	    message("[Previously saved workspace restored]", "\n")
     }
