@@ -1,7 +1,7 @@
 #  File src/library/base/R/match.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2016 The R Core Team
+#  Copyright (C) 1995-2022 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -34,13 +34,16 @@ pmatch <- function(x, table, nomatch = NA_integer_, duplicates.ok = FALSE)
     .Internal(pmatch(as.character(x), as.character(table), nomatch,
                      duplicates.ok))
 
+# "utils::hasName(x, name)" is defined to be the same as "name %in% names(x)",
+# so change it if this changes.
 `%in%`  <- function(x, table) match(x, table, nomatch = 0L) > 0L
 
 match.arg <- function (arg, choices, several.ok = FALSE)
 {
     if (missing(choices)) {
-	formal.args <- formals(sys.function(sys.parent()))
-	choices <- eval(formal.args[[as.character(substitute(arg))]])
+	formal.args <- formals(sys.function(sysP <- sys.parent()))
+	choices <- eval(formal.args[[as.character(substitute(arg))]],
+			envir = sys.frame(sysP))
     }
     if (is.null(arg)) return(choices[1L])
     else if(!is.character(arg))
@@ -54,11 +57,13 @@ match.arg <- function (arg, choices, several.ok = FALSE)
     ## handle each element of arg separately
     i <- pmatch(arg, choices, nomatch = 0L, duplicates.ok = TRUE)
     if (all(i == 0L))
-	stop(gettextf("'arg' should be one of %s",
-                      paste(dQuote(choices), collapse = ", ")),
+        stop(sprintf(ngettext(length(chs <- unique(choices[nzchar(choices)])),
+                              "'arg' should be %s",
+                              "'arg' should be one of %s"),
+                     paste(dQuote(chs), collapse=", ")),
              domain = NA)
     i <- i[i > 0L]
-    if (!several.ok && length(i) > 1)
+    if (!several.ok && length(i) > 1) ## can this happen ??
         stop("there is more than one match in 'match.arg'")
     choices[i]
 }
@@ -76,3 +81,12 @@ char.expand <- function(input, target, nomatch = stop("no match"))
     if(anyNA(y)) eval(nomatch)
     target[y]
 }
+
+mtfrm <- function(x)
+    UseMethod("mtfrm")
+
+mtfrm.default <- function(x) {
+    if(length(y <- as.character(x)) != length(x))
+        stop("cannot mtfrm")
+    y
+}
