@@ -1,7 +1,7 @@
 #  File src/library/splines/R/splines.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2017 The R Core Team
+#  Copyright (C) 1995-2023 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 #  https://www.R-project.org/Licenses/
 
 bs <- function(x, df = NULL, knots = NULL, degree = 3, intercept = FALSE,
-               Boundary.knots = range(x))
+               Boundary.knots = range(x), warn.outside = TRUE)
 {
     ord <- 1L + (degree <- as.integer(degree))
     if(ord <= 1) stop("'degree' must be integer >= 1")
@@ -31,7 +31,7 @@ bs <- function(x, df = NULL, knots = NULL, degree = 3, intercept = FALSE,
         (ol <- x < Boundary.knots[1L]) | (or <- x > Boundary.knots[2L])
     } else FALSE
 
-    if(!is.null(df) && is.null(knots)) {
+    if(mk.knots <- !is.null(df) && is.null(knots)) {
 	nIknots <- df - ord + (1L - intercept) # ==  #{inner knots}
         if(nIknots < 0L) {
             nIknots <- 0L
@@ -42,12 +42,31 @@ bs <- function(x, df = NULL, knots = NULL, degree = 3, intercept = FALSE,
             if(nIknots > 0L) {
                 knots <- seq.int(from = 0, to = 1,
                                  length.out = nIknots + 2L)[-c(1L, nIknots + 2L)]
-                quantile(x[!outside], knots)
+                quantile(x[!outside], knots, names=FALSE)
             }
     }
+    else if(!all(is.finite(knots))) stop("non-finite knots")
+    if(mk.knots && length(knots) && any(lrEq <- range(knots) %in% Boundary.knots)) {
+        if(lrEq[1L]) {
+            aE <- all(i <- knots == (piv <- Boundary.knots[1L]))
+            if(aE)
+                warning("all interior knots match left boundary knot")
+            else
+                knots[i] <- knots[i] + (min(knots[knots > piv]) - piv)/8
+        }
+        if(lrEq[2L]) {
+            aE2 <- all(i <- knots == (piv <- Boundary.knots[2L]))
+            if(aE2)
+                warning("all interior knots match right boundary knot")
+            else
+                knots[i] <- knots[i] - (piv - max(knots[knots < piv]))/8
+        }
+        if(!(lrEq[1L] && aE || lrEq[2L] && aE2)) # haven't warned yet
+            warning("shoving 'interior' knots matching boundary knots to inside")
+    }
     Aknots <- sort(c(rep(Boundary.knots, ord), knots))
     if(any(outside)) {
-        warning("some 'x' values beyond boundary knots may cause ill-conditioned bases")
+        if(warn.outside) warning("some 'x' values beyond boundary knots may cause ill-conditioned bases")
         derivs <- 0:degree
         scalef <- gamma(1L:ord)# factorials
         basis <- array(0, c(length(x), length(Aknots) - degree - 1L))
@@ -55,14 +74,14 @@ bs <- function(x, df = NULL, knots = NULL, degree = 3, intercept = FALSE,
         if(any(ol)) {
 	    ## left pivot inside, i.e., a bit to the right of the boundary knot
 	    k.pivot <- (1-e)*Boundary.knots[1L] + e*Aknots[ord+1]
-            xl <- cbind(1, outer(x[ol] - k.pivot, 1L:degree, "^"))
+            xl <- cbind(1, outer(x[ol] - k.pivot, 1L:degree, `^`))
             tt <- splineDesign(Aknots, rep(k.pivot, ord), ord, derivs)
             basis[ol, ] <- xl %*% (tt/scalef)
         }
         if(any(or)) {
 	    ## right pivot inside, i.e., a bit to the left of the boundary knot:
 	    k.pivot <- (1-e)*Boundary.knots[2L] + e*Aknots[length(Aknots)-ord]
-            xr <- cbind(1, outer(x[or] - k.pivot, 1L:degree, "^"))
+            xr <- cbind(1, outer(x[or] - k.pivot, 1L:degree, `^`))
             tt <- splineDesign(Aknots, rep(k.pivot, ord), ord, derivs)
             basis[or, ] <- xr %*% (tt/scalef)
         }
@@ -103,7 +122,7 @@ ns <- function(x, df = NULL, knots = NULL, intercept = FALSE,
 	    Boundary.knots <- x*c(7,9)/8 # symmetrically around x
 	FALSE # rep(FALSE, length = length(x))
     }
-    if(!is.null(df) && is.null(knots)) {
+    if(mk.knots <- !is.null(df) && is.null(knots)) {
         ## df = number(interior knots) + 1 + intercept
         nIknots <- df - 1L - intercept
         if(nIknots < 0L) {
@@ -115,9 +134,25 @@ ns <- function(x, df = NULL, knots = NULL, intercept = FALSE,
             if(nIknots > 0L) {
                 knots <- seq.int(from = 0, to = 1,
                                  length.out = nIknots + 2L)[-c(1L, nIknots + 2L)]
-                quantile(x[!outside], knots)
+                quantile(x[!outside], knots, names=FALSE)
             }
-    } else nIknots <- length(knots)
+    } else {
+        if(!all(is.finite(knots))) stop("non-finite knots")
+        nIknots <- length(knots)
+    }
+    if(mk.knots && length(knots) && any(lrEq <- range(knots) %in% Boundary.knots)) {
+        if(lrEq[1L]) {
+            i <- knots == (piv <- Boundary.knots[1L])
+            if(all(i)) stop("all interior knots match left boundary knot")
+            knots[i] <- knots[i] + (min(knots[knots > piv]) - piv)/8
+        }
+        if(lrEq[2L]) {
+            i <- knots == (piv <- Boundary.knots[2L])
+            if(all(i)) stop("all interior knots match right boundary knot")
+            knots[i] <- knots[i] - (piv - max(knots[knots < piv]))/8
+        }
+        warning("shoving 'interior' knots matching boundary knots to inside")
+    }
     Aknots <- sort(c(rep(Boundary.knots, 4L), knots))
     if(any(outside)) {
         basis <- array(0, c(length(x), nIknots + 4L))
@@ -180,20 +215,23 @@ predict.ns <- function(object, newx, ...)
 
 makepredictcall.ns <- function(var, call)
 {
-    if(as.character(call)[1L] != "ns") return(call)
-    at <- attributes(var)[c("knots", "Boundary.knots", "intercept")]
-    xxx <- call[1L:2L]
-    xxx[names(at)] <- at
-    xxx
+    ## check must work correctly when call is a symbol, both for quote(ns) and quote(t1):
+    if(as.character(call)[1L] == "ns" || (is.call(call) && identical(eval(call[[1L]]), ns))) {
+	at <- attributes(var)[c("knots", "Boundary.knots", "intercept")]
+	call <- call[1L:2L]
+	call[names(at)] <- at
+    }
+    call
 }
 
 makepredictcall.bs <- function(var, call)
 {
-    if(as.character(call)[1L] != "bs") return(call)
-    at <- attributes(var)[c("degree", "knots", "Boundary.knots", "intercept")]
-    xxx <- call[1L:2]
-    xxx[names(at)] <- at
-    xxx
+    if(as.character(call)[1L] == "bs" || (is.call(call) && identical(eval(call[[1L]]), bs))) {
+	at <- attributes(var)[c("degree", "knots", "Boundary.knots", "intercept")]
+	call <- call[1L:2L]
+	call[names(at)] <- at
+    }
+    call
 }
 
 
