@@ -1,8 +1,8 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
+ *  Copyright (C) 1998-2022   The R Core Team.
+ *  Copyright (C) 2004-2017   The R Foundation
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1998-2014   The R Core Team.
- *  Copyright (C) 2004-5        The R Foundation
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -60,6 +60,22 @@ static SEXP LGammaSymbol;
 static SEXP DiGammaSymbol;
 static SEXP TriGammaSymbol;
 static SEXP PsiSymbol;
+/* new symbols in R 3.4.0: */
+static SEXP PiSymbol;
+static SEXP ExpM1Symbol;
+static SEXP Log1PSymbol;
+static SEXP Log2Symbol;
+static SEXP Log10Symbol;
+static SEXP SinPiSymbol;
+static SEXP CosPiSymbol;
+static SEXP TanPiSymbol;
+static SEXP FactorialSymbol;
+static SEXP LFactorialSymbol;
+/* possible future symbols
+static SEXP Log1PExpSymbol;
+static SEXP Log1MExpSymbol;
+static SEXP Log1PMxSymbol;
+*/
 
 static Rboolean Initialized = FALSE;
 
@@ -93,6 +109,22 @@ static void InitDerivSymbols(void)
     DiGammaSymbol = install("digamma");
     TriGammaSymbol = install("trigamma");
     PsiSymbol = install("psigamma");
+/* new symbols */
+    PiSymbol = install("pi");
+    ExpM1Symbol = install("expm1");
+    Log1PSymbol = install("log1p");
+    Log2Symbol = install("log2");
+    Log10Symbol = install("log10");
+    SinPiSymbol = install("sinpi");
+    CosPiSymbol = install("cospi");
+    TanPiSymbol = install("tanpi");
+    FactorialSymbol = install("factorial");
+    LFactorialSymbol = install("lfactorial");
+/* possible future symbols
+    Log1PExpSymbol = install("log1pexp");    # log(1+exp(x))
+    Log1MExpSymbol = install("log1mexp");    # log(1-exp(-x)), for x > 0
+    Log1PMxSymbol = install("log1pmx");      # log1p(x)-x
+*/
 
     Initialized = TRUE;
 }
@@ -235,16 +267,18 @@ static SEXP simplify(SEXP fun, SEXP arg1, SEXP arg2)
 	    ans = lang3(PowerSymbol, arg1, arg2);
     }
     else if (fun == ExpSymbol) {
-	/* FIXME: simplify exp(lgamma( E )) = gamma( E ) */
-	ans = lang2(ExpSymbol, arg1);
+        /* FIXME: simplify exp(lgamma( E )) = gamma( E ) */
+        /* FIXME: simplify exp(lfactorial( E )) = factorial( E ) */
+        ans = lang2(ExpSymbol, arg1);
     }
     else if (fun == LogSymbol) {
-	/* FIXME: simplify log(gamma( E )) = lgamma( E ) */
-	ans = lang2(LogSymbol, arg1);
+        /* FIXME: simplify log(gamma( E )) = lgamma( E ) */
+        /* FIXME: simplify log(factorial( E )) = lfactorial( E ) */
+        ans = lang2(LogSymbol, arg1);
     }
-    else if (fun == CosSymbol)	ans = lang2(CosSymbol, arg1);
-    else if (fun == SinSymbol)	ans = lang2(SinSymbol, arg1);
-    else if (fun == TanSymbol)	ans = lang2(TanSymbol, arg1);
+    else if (fun == CosSymbol)  ans = lang2(CosSymbol, arg1);
+    else if (fun == SinSymbol)  ans = lang2(SinSymbol, arg1);
+    else if (fun == TanSymbol)  ans = lang2(TanSymbol, arg1);
     else if (fun == CoshSymbol) ans = lang2(CoshSymbol, arg1);
     else if (fun == SinhSymbol) ans = lang2(SinhSymbol, arg1);
     else if (fun == TanhSymbol) ans = lang2(TanhSymbol, arg1);
@@ -262,6 +296,28 @@ static SEXP simplify(SEXP fun, SEXP arg1, SEXP arg2)
        if (arg2 == R_MissingArg) ans = lang2(PsiSymbol, arg1);
        else ans = lang3(PsiSymbol, arg1, arg2);
     }
+/* new symbols */
+    else if (fun == ExpM1Symbol) {
+        /* FIXME: simplify expm1(log1p( E )) = E */
+        ans = lang2(ExpM1Symbol, arg1);
+    }
+    else if (fun == LogSymbol) {
+        /* FIXME: simplify log1p(expm1( E )) = E */
+        ans = lang2(Log1PSymbol, arg1);
+    }
+    else if (fun == Log2Symbol) ans = lang2(Log2Symbol, arg1);
+    else if (fun == Log10Symbol) ans = lang2(Log10Symbol, arg1);
+    else if (fun == CosPiSymbol) ans = lang2(CosPiSymbol, arg1);
+    else if (fun == SinPiSymbol) ans = lang2(SinPiSymbol, arg1);
+    else if (fun == TanPiSymbol) ans = lang2(TanPiSymbol, arg1);
+    else if (fun == FactorialSymbol)ans = lang2(FactorialSymbol, arg1);
+    else if (fun == LFactorialSymbol)ans = lang2(LFactorialSymbol, arg1);
+/* possible future symbols
+    else if (fun == Log1PExpSymbol) ans = lang2(Log1PExpSymbol, arg1);
+    else if (fun == Log1MExpSymbol) ans = lang2(Log1MExpSymbol, arg1);
+    else if (fun == Log1PMxSymbol) ans = lang2(Log1PMxSymbol, arg1);
+*/
+
     else ans = Constant(NA_REAL);
     /* FIXME */
 #ifdef NOTYET
@@ -383,7 +439,8 @@ static SEXP D(SEXP expr, SEXP var)
 	}
 	else if (CAR(expr) == LogSymbol) {
 	    if (length(expr) != 2)
-		error("only single-argument calls are supported");
+		error("only single-argument calls to log() are supported;\n"
+		      "  maybe use log(x,a) = log(x)/log(a)");
 	    ans = simplify(DivideSymbol,
 			   PP(D(CADR(expr), var)),
 			   CADR(expr));
@@ -466,7 +523,7 @@ static SEXP D(SEXP expr, SEXP var)
 			   PP_S(DivideSymbol,
 				PP(D(CADR(expr), var)),
 				PP_S(SqrtSymbol,
-				     PP_S(MinusSymbol,PP(Constant(1.)),
+				     PP_S(MinusSymbol, PP(Constant(1.)),
 					  PP_S(PowerSymbol,
 					       CADR(expr),PP(Constant(2.)))),
 				     R_MissingArg)), R_MissingArg);
@@ -529,6 +586,96 @@ static SEXP D(SEXP expr, SEXP var)
 		UNPROTECT(3);
 	    }
 	}
+/* new in R 3.4.0 */
+        else if (CAR(expr) == ExpM1Symbol) {
+            ans = simplify(TimesSymbol,
+			   PP_S2(ExpSymbol, CADR(expr)),
+                           PP(D(CADR(expr), var)));
+            UNPROTECT(2);
+        }
+        else if (CAR(expr) == Log1PSymbol) {
+            ans = simplify(DivideSymbol,
+                           PP(D(CADR(expr), var)),
+                           PP_S(PlusSymbol, PP(Constant(1.)), CADR(expr)));
+            UNPROTECT(3);
+        }
+        else if (CAR(expr) == Log2Symbol) {
+            ans = simplify(DivideSymbol,
+                           PP(D(CADR(expr), var)),
+                           PP_S(TimesSymbol, CADR(expr),
+				             PP_S2(LogSymbol, PP(Constant(2.)))));
+            UNPROTECT(4);
+        }
+        else if (CAR(expr) == Log10Symbol) {
+            ans = simplify(DivideSymbol,
+                           PP(D(CADR(expr), var)),
+                           PP_S(TimesSymbol, CADR(expr),
+				             PP_S2(LogSymbol, PP(Constant(10.)))));
+            UNPROTECT(4);
+        }
+        else if (CAR(expr) == CosPiSymbol) {
+            ans = simplify(TimesSymbol,
+                           PP_S2(SinPiSymbol, CADR(expr)),
+                           PP_S(TimesSymbol, PP_S2(MinusSymbol, PiSymbol),
+				             PP(D(CADR(expr), var)) ));
+            UNPROTECT(4);
+        }
+        else if (CAR(expr) == SinPiSymbol) {
+            ans = simplify(TimesSymbol,
+                           PP_S2(CosPiSymbol, CADR(expr)),
+                           PP_S(TimesSymbol, PiSymbol,
+                                             PP(D(CADR(expr), var)) ));
+            UNPROTECT(3);
+        }
+        else if (CAR(expr) == TanPiSymbol) {
+            ans = simplify(DivideSymbol,
+                           PP_S(TimesSymbol, PiSymbol, PP(D(CADR(expr), var))),
+			   PP_S(PowerSymbol,
+				PP_S2(CosPiSymbol, CADR(expr)),
+				PP(Constant(2.0))));
+            UNPROTECT(5);
+        }
+        else if (CAR(expr) == LFactorialSymbol) {
+            ans = simplify(TimesSymbol,
+                           PP(D(CADR(expr), var)),
+                           PP_S2(DiGammaSymbol, PP_S(PlusSymbol,
+						     CADR(expr),
+						     PP(ScalarInteger(1)))));
+            UNPROTECT(4);
+        }
+        else if (CAR(expr) == FactorialSymbol) {
+            ans = simplify(TimesSymbol,
+                           PP(D(CADR(expr), var)),
+                           PP_S(TimesSymbol,
+                                expr,
+                                PP_S2(DiGammaSymbol, PP_S(PlusSymbol,
+							  CADR(expr),
+							  PP(ScalarInteger(1))))));
+            UNPROTECT(5);
+        }
+/* possible future symbols
+        else if (CAR(expr) == Log1PExpSymbol) {
+            ans = simplify(DivideSymbol,
+                           PP_S(TimesSymbol, PP(D(CADR(expr), var)),
+                                PP_S2(ExpSymbol, CADR(expr))),
+                           PP_S(PlusSymbol,PP(Constant(1.)),
+                                PP_S2(ExpSymbol, CADR(expr)) ));
+            UNPROTECT(6);
+        }
+        else if (CAR(expr) == Log1MExpSymbol) {
+            ans = simplify(DivideSymbol,
+                           PP_S(TimesSymbol, PP_S2(MinusSymbol, PP(D(CADR(expr), var))),
+                                PP_S2(ExpSymbol, PP_S2(MinusSymbol, CADR(expr))) ),
+                           PP_S2(ExpM1Symbol, PP_S2(MinusSymbol, CADR(expr))) );
+            UNPROTECT(7);
+        }
+        else if (CAR(expr) == Log1PMxSymbol) {
+            ans = simplify(DivideSymbol,
+                           PP_S2(MinusSymbol, PP(D(CADR(expr), var))),
+                           PP_S(PlusSymbol,PP(Constant(1.)), CADR(expr)) );
+            UNPROTECT(4);
+        }
+*/
 
 	else {
 	    SEXP u = deparse1(CAR(expr), 0, SIMPLEDEPARSE);
@@ -543,6 +690,7 @@ static SEXP D(SEXP expr, SEXP var)
     return ans;
 
 #undef PP_S
+#undef PP_S2
 
 } /* D() */
 
@@ -638,7 +786,7 @@ SEXP doD(SEXP args)
     if (isExpression(CAR(args))) expr = VECTOR_ELT(CAR(args), 0);
     else expr = CAR(args);
     if (!(isLanguage(expr) || isSymbol(expr) || isNumeric(expr) || isComplex(expr)))
-        error(_("'expr' must be an expression or call"));
+        error(_("expression must not be type '%s'"), type2char(TYPEOF(expr)));
     var = CADR(args);
     if (!isString(var) || length(var) < 1)
 	error(_("variable must be a character string"));
@@ -654,7 +802,7 @@ SEXP doD(SEXP args)
 
 /* ------ FindSubexprs ------ and ------ Accumulate ------ */
 
-static void NORET InvalidExpression(char *where)
+NORET static void InvalidExpression(char *where)
 {
     error(_("invalid expression in '%s'"), where);
 }
@@ -716,16 +864,16 @@ static int Accumulate2(SEXP expr, SEXP exprlist)
     return k + 1;
 }
 
-static SEXP MakeVariable(int k, SEXP tag)
+static SEXP MakeVariable(int k, const char *tag)
 {
-    const void *vmax = vmaxget();    
     char buf[64];
-    snprintf(buf, 64, "%s%d", translateChar(STRING_ELT(tag, 0)), k);
-    vmaxset(vmax);
+    int res = snprintf(buf, 64, "%s%d", tag, k);
+    if (res >= 64)
+	error(_("too many variables"));
     return install(buf);
 }
 
-static int FindSubexprs(SEXP expr, SEXP exprlist, SEXP tag)
+static int FindSubexprs(SEXP expr, SEXP exprlist, const char *tag)
 {
     SEXP e;
     int k;
@@ -921,7 +1069,9 @@ SEXP deriv(SEXP args)
     SEXP ans, ans2, expr, funarg, names, s;
     int f_index, *d_index, *d2_index;
     int i, j, k, nexpr, nderiv=0, hessian;
-    SEXP exprlist, tag;
+    SEXP exprlist, stag;
+    const void *vmax = vmaxget();
+    const char *tag;
 
     args = CDR(args);
     InitDerivSymbols();
@@ -940,10 +1090,12 @@ SEXP deriv(SEXP args)
     funarg = CAR(args);
     args = CDR(args);
     /* tag: */
-    tag = CAR(args);
-    if (!isString(tag) || length(tag) < 1
-	|| length(STRING_ELT(tag, 0)) < 1 || length(STRING_ELT(tag, 0)) > 60)
+    stag = CAR(args);
+    if (!isString(stag) || length(stag) < 1 || length(STRING_ELT(stag, 0)) < 1)
 	error(_("invalid tag"));
+    tag = translateChar(STRING_ELT(stag, 0));
+    if(strlen(tag) > 60) error(_("invalid tag"));
+
     args = CDR(args);
     /* hessian: */
     hessian = asLogical(CAR(args));
@@ -1114,6 +1266,7 @@ SEXP deriv(SEXP args)
 	SET_VECTOR_ELT(funarg, 0, exprlist);
 	/* funarg = lang2(install("expression"), exprlist); */
     }
+    vmaxset(vmax);
     UNPROTECT(2);
     return funarg;
 }
