@@ -1,8 +1,8 @@
 #  File src/library/utils/R/completion.R
 #  Part of the R package, https://www.R-project.org
 #
-# Copyright     2006 Deepayan Sarkar
-#           (C) 2006-2017  The R Core Team
+# Copyright (C) 2006-2022  The R Core Team
+# Copyright (C) 2006-2022  Deepayan Sarkar
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -44,6 +44,15 @@
 ### so will change all pastes to sprintf.  However, we need to be
 ### careful because 0 length components in sprintf will cause errors.
 
+## Should compare with paste0() and if you compare with a pure format string, it may change:
+if(FALSE) {
+    ii <- seq_len(1e5) ; ilet <- seq_along(letters)
+    system.time(for (i in ii) sprintf("%s%s%s%d", "foo", letters, "bar", ilet) )[1] # 1.101
+    system.time(for (i in ii) paste0(             "foo", letters, "bar", ilet) )[1] # 0.936
+    ## But for the most frequent case,  sprintf() is still faster:
+    system.time(for (i in ii) sprintf("^%s", letters)) # 0.344
+    system.time(for (i in ii) paste0 ("^",   letters)) # 0.444
+}
 
 ## [July 2013] First attempt to support fuzzy matching, if
 ## rc.settings(fuzzy=TRUE), based on suggestion from Rasmus Baath.
@@ -90,7 +99,7 @@ findExactMatches <- function(pattern, values)
 
 findFuzzyMatches <- function(pattern, values) {
     ## FIXME: option to allow experimentation, remove eventually
-    if (!is.null(ffun <- getOption("fuzzy.match.fun"))) {
+    if (!is.null(ffun <- rc.getOption("fuzzy.match.fun"))) {
         return (ffun(pattern, values))
     }
     ## Try exact matches first, and return them if found
@@ -111,9 +120,10 @@ findFuzzyMatches <- function(pattern, values) {
 }
 
 
-findMatches <- function(pattern, values)
+findMatches <- function(pattern, values, fuzzy)
 {
-    if (.CompletionEnv$settings[["fuzzy"]])
+    if (missing(fuzzy)) fuzzy <- .CompletionEnv$settings[["fuzzy"]]
+    if (fuzzy)
         findFuzzyMatches(pattern, values)
     else
         findExactMatches(pattern, values)
@@ -151,6 +161,18 @@ fuzzyApropos <- function(what)
         findMatches(pattern, ls(x, all.names = TRUE))
 }
 
+## generic and default method to generate completion after @
+
+.AtNames <- function(x, pattern)
+    UseMethod(".AtNames")
+
+.AtNames.default <- function(x, pattern = "") {
+    if (isS4(x))
+        findMatches(pattern, methods::slotNames(x))
+    else
+        character()
+}
+
 ## if (is.environment(object))
 ## {
 ##     ls(object,
@@ -168,7 +190,7 @@ fuzzyApropos <- function(what)
 
 ## modifies settings:
 
-rc.settings <- function(ops, ns, args, func, ipck, S3, data, help, argdb, fuzzy, quotes, files)
+rc.settings <- function(ops, ns, args, dots, func, ipck, S3, data, help, argdb, fuzzy, quotes, files)
 {
     if (length(match.call()) == 1) return(unlist(.CompletionEnv[["settings"]]))
     checkAndChange <- function(what, value)
@@ -181,6 +203,7 @@ rc.settings <- function(ops, ns, args, func, ipck, S3, data, help, argdb, fuzzy,
     if (!missing(ops))   checkAndChange(  "ops",   ops)
     if (!missing(ns))    checkAndChange(   "ns",    ns)
     if (!missing(args))  checkAndChange( "args",  args)
+    if (!missing(dots))  checkAndChange( "dots",  dots)
     if (!missing(func))  checkAndChange( "func",  func)
     if (!missing(ipck))  checkAndChange( "ipck",  ipck)
     if (!missing(S3))    checkAndChange(   "S3",    S3)
@@ -332,14 +355,14 @@ makeRegexpSafe <- function(s)
 ## Operators that are handled specially.  Order is important, ::: must
 ## come before :: (because :: will match :::)
 
-specialOps <- c("$", "@", ":::", "::", "?", "[", "[[")
+specialOps <- c("$", "@", ":::", "::", "?", "[[", "[")
 
 
 specialOpCompletionsHelper <- function(op, suffix, prefix)
 {
     tryToEval <- function(s)
     {
-	tryCatch(eval(parse(text = s), envir = .GlobalEnv), error = function(e)e)
+	tryCatch(eval(str2expression(s), envir = .GlobalEnv), error = function(e)e)
     }
     switch(op,
            "$" = {
@@ -350,7 +373,6 @@ specialOpCompletionsHelper <- function(op, suffix, prefix)
                        suffix
                    else
                    {
-                       ## ## suffix must match names(object) (or ls(object) for environments)
                        .DollarNames(object, pattern = sprintf("^%s", makeRegexpSafe(suffix)))
                    }
                } else suffix
@@ -363,15 +385,19 @@ specialOpCompletionsHelper <- function(op, suffix, prefix)
                        suffix
                    else
                    {
-                       findMatches(sprintf("^%s", makeRegexpSafe(suffix)),
-                                   methods::slotNames(object))
+                       .AtNames(object, pattern = sprintf("^%s", makeRegexpSafe(suffix)))
                    }
                } else suffix
            },
            "::" = {
                if (.CompletionEnv$settings[["ns"]])
                {
-                   nse <- tryCatch(getNamespaceExports(prefix), error = function(e)e)
+                   nse <- tryCatch(unique(c(getNamespaceExports(prefix),
+                                            if(prefix != "base")
+                                                names(getNamespaceInfo(prefix,
+                                                                       "lazydata"))
+                                            )),
+                                   error = identity)
                    if (inherits(nse, "error")) ## nothing else to do
                        suffix
                    else
@@ -443,7 +469,7 @@ matchAvailableTopics <- function(prefix, text)
     }
     if (length(text) != 1L || text == "") return (character())
     ## Update list of help topics if necessary
-    pkgpaths <- searchpaths()[substr(search(), 1L, 8L) == "package:"]
+    pkgpaths <- searchpaths()[startsWith(search(), "package:")]
     if (!identical(basename(pkgpaths), .CompletionEnv[["attached_packages"]])) {
         assign("attached_packages",
                basename(pkgpaths),
@@ -604,7 +630,8 @@ normalCompletions <-
             if (.CompletionEnv$settings[["fuzzy"]])
                 fuzzyApropos(sprintf("^%s", makeRegexpSafe(text)))
             else
-                apropos(sprintf("^%s", makeRegexpSafe(text)), ignore.case = FALSE)
+                apropos(sprintf("^%s", makeRegexpSafe(text)), ignore.case = FALSE,
+                        dot_internals = TRUE)
         if (.CompletionEnv$settings[["func"]] && check.mode && !is.null(add.fun))
         {
             which.function <- sapply(comps, function(s) exists(s, mode = "function"))
@@ -682,7 +709,7 @@ inFunction <-
 
     temp <-
         data.frame(i = c(parens[["("]], parens[[")"]]),
-                   c = rep(c(1, -1), lengths(parens)))
+                   c = rep.int(c(1, -1), lengths(parens)))
     if (nrow(temp) == 0) return(character())
     temp <- temp[order(-temp$i), , drop = FALSE] ## order backwards
     wp <- which(cumsum(temp$c) > 0)
@@ -779,11 +806,11 @@ specialFunctionArgs <- function(fun, text)
 }
 
 
-
 functionArgs <-
     function(fun, text,
              S3methods = .CompletionEnv$settings[["S3"]],
              S4methods = FALSE,
+             keep.dots = .CompletionEnv$settings[["dots"]],
              add.args = rc.getOption("funarg.suffix"))
 {
     if (length(fun) < 1L || any(fun == "")) return(character())
@@ -797,8 +824,16 @@ functionArgs <-
     if (S4methods) warning("cannot handle S4 methods yet")
     allArgs <- unique(unlist(lapply(fun, argNames)))
     ans <- findMatches(sprintf("^%s", makeRegexpSafe(text)), allArgs)
+    ## Handle dots specially. If present, drop if keep.dots=FALSE
+    ## obviously, but even otherwise move it to the end and don't add
+    ## a suffix.
+    whereDots <- ans == "..."
+    if (any(whereDots))
+        ans <- ans[!whereDots]
     if (length(ans) && !is.null(add.args))
         ans <- sprintf("%s%s", ans, add.args)
+    if (keep.dots && any(whereDots))
+        ans <- c(ans, "...")
     c(specialFunArgs, ans)
 }
 
@@ -849,7 +884,6 @@ fileCompletionPreferred <- function()
         lbss <- head.default(unlist(strsplit(linebuffer, "")), .CompletionEnv[["end"]])
         ((sum(lbss == "'") %% 2 == 1) ||
          (sum(lbss == '"') %% 2 == 1))
-
     })
 }
 
@@ -899,10 +933,10 @@ fileCompletions <- function(token)
     ## are included.  Get 'correct' partial file name by looking back
     ## to begin quote
     pfilename <- correctFilenameToken()
-    
     ## This may come from an illegal string like "C:\Prog".  Try to parse it:
-    pfilename <- try(eval(parse(text = paste0('"', token, '"'))), silent = TRUE)
-    if (inherits(pfilename, "try-error"))
+    pfilename <- tryCatch(eval(str2expression(paste0('"', token, '"'))),
+                          error = identity)
+    if (inherits(pfilename, "error"))
     	return(character())
 
     ## Sys.glob doesn't work without expansion.  Is that intended?
@@ -916,7 +950,7 @@ fileCompletions <- function(token)
     ## actually looking for something inside the directory.  Note that
     ## we don't actually test to see if it's a directory, because if
     ## it is not, list.files() will simply return character(0).
-    if (length(comps) == 1 && substring(comps, nchar(comps), nchar(comps)) == "/") {
+    if (length(comps) == 1 && endsWith(comps, "/")) {
         filesInside <- list.files(comps, all.files = TRUE, full.names = FALSE, no.. = TRUE)
         if (length(filesInside)) comps <- c(comps, file.path(comps, filesInside))
     }
@@ -930,11 +964,11 @@ fileCompletions <- function(token)
     ## need to delete extra part
     if (pfilename != token)
         comps <- substring(comps, nchar(pfilename) - nchar(token) + 1L, 1000L)
-        
-    ## In Win32, we often have backslashes in names. Also possible on 
+
+    ## In Win32, we often have backslashes in names. Also possible on
     ## Unix, though unlikely.  Add escapes for those and for quotes.
     comps <- gsub("([\\\\'\"])", "\\\\\\1", comps)
-    
+
     comps
 }
 
@@ -1061,8 +1095,11 @@ fileCompletions <- function(token)
                     substring(helpCompletions(fullToken$prefix, fullToken$token),
                               2L + nchar(fullToken$prefix), 1000L)    # drop initial "prefix + ?"
                 }
-                else if (!probablySpecial)
+                else if (!probablySpecial) # probably a filename
                     fileCompletions(fullToken$token) # FIXME: but not if probablyBacktick
+                ## Give a custom completer function a chance to add to this list
+                else if (is.function(fcustomQuote <- rc.getOption("custom.quote.completer")))
+                    fcustomQuote(fullToken)
             .setFileComp(FALSE)
             ## str(c(fullToken, list(comps = tentativeCompletions)))
             ## Adjust for self-computed token
@@ -1144,6 +1181,9 @@ fileCompletions <- function(token)
             comps <- paste0(prefix, comps)
         }
         comps <- c(fargComps, comps)
+        ## Give a custom completer function a chance to add to this list
+        if (is.function(fcustom <- rc.getOption("custom.argument.completer")))
+            comps <- c(fcustom(guessedFunction, text), comps)
         .CompletionEnv[["comps"]] <- comps
     }
 }
@@ -1312,26 +1352,52 @@ fileCompletions <- function(token)
           "lty", "lwd", "mai", "mar", "mex", "mfcol", "mfg", "mfrow",
           "mgp", "mkh", "new", "oma", "omd", "omi", "pch", "pin",
           "plt", "ps", "pty", "smo", "srt", "tck", "tcl", "usr",
-          "xaxp", "xaxs", "xaxt", "xpd", "yaxp", "yaxs", "yaxt")
+          "xaxp", "xaxs", "xaxt", "xpd", "yaxp", "yaxs", "yaxt",
+          "page", "ylbias")
 
-    options <- c("add.smooth", "browser", "check.bounds", "continue",
-	"contrasts", "defaultPackages", "demo.ask", "device",
-	"digits", "dvipscmd", "echo", "editor", "encoding",
-	"example.ask", "expressions", "help.search.types",
-	"help.try.all.packages", "htmlhelp", "HTTPUserAgent",
-	"internet.info", "keep.source", "keep.source.pkgs",
-	"locatorBell", "mailer", "max.print", "menu.graphics",
-	"na.action", "OutDec", "pager", "papersize",
-	"par.ask.default", "pdfviewer", "pkgType", "printcmd",
-	"prompt", "repos", "scipen", "show.coef.Pvalues",
-	"show.error.messages", "show.signif.stars", "str",
-	"stringsAsFactors", "timeout", "ts.eps", "ts.S.compat",
-	"unzip", "verbose", "warn", "warning.length", "width")
+    options <- unique(c(
+        names(.Options),
+        ## + options not yet initialized when preparing utils
+        "bitmapType", "citation.bibtex.max", "contrasts", "demo.ask",
+        "device", "device.ask.default", "editor", "example.ask",
+        "help.search.types", "help.try.all.packages", "HTTPUserAgent",
+        "internet.info", "locatorBell", "mailer", "menu.graphics",
+        "na.action", "pkgType", "repos", "show.coef.Pvalues",
+        "show.signif.stars", "str", "str.dendrogram.last",
+        "ts.eps", "ts.S.compat", "unzip", "windowsTimeout",
+        ## + options unset by default (or OS-specific)
+        "mc.cores", "dvipscmd", "warn.FPU",
+        "askYesNo", "BioC_mirror", "ccaddress", "checkPackageLicense",
+        "conflicts.policy", "de.cellwidth", "deparse.max.lines", "digits.secs",
+        "download.file.extra", "download.file.method", "error",
+        "help.htmlmath", "help.ports", "help_type", "install.lock",
+        "install.packages.check.source",
+        "install.packages.compile.from.source",
+        "interrupt", "Ncpus", "save.defaults", "save.image.defaults",
+        "setWidthOnResize", "show.error.locations", "show.nls.convergence",
+        "SweaveHooks", "SweaveSyntax", "topLevelEnvironment",
+        "traceback.max.lines", "url.method", "warning.expression"
+    ))
 
     .addFunctionInfo(par = par, options = options)
 
     ## read.csv etc (... passed to read.table)
+    readTable <-
+        c("file", "header", "sep", "quote", "dec", "numerals",
+          "row.names", "col.names", "as.is", "na.strings",
+          "colClasses", "nrows", "skip", "check.names", "fill",
+          "strip.white", "blank.lines.skip", "comment.char",
+          "allowEscapes", "flush", "stringsAsFactors", "fileEncoding",
+          "encoding", "text", "skipNul")
 
+    .addFunctionInfo(read.csv = readTable, read.csv2 = readTable,
+                     read.delim = readTable, read.delim2 = readTable)
+
+    ## c() because it is primitive c(...) and there is no proper
+    ## c.default even though docs say it is
+    ## c.default(..., recursive = FALSE, use.names = TRUE)
+
+    .addFunctionInfo(c = c("recursive", "use.names"))
 }
 
 
@@ -1339,28 +1405,28 @@ fileCompletions <- function(token)
 .CompletionEnv <- new.env(hash = FALSE)
 
 ## needed to save some overhead in .win32consoleCompletion
-assign("linebuffer", "", env = .CompletionEnv)
-assign("end", 1, env = .CompletionEnv)
+assign("linebuffer", "", envir = .CompletionEnv)
+assign("end", 1, envir = .CompletionEnv)
 
 assign("settings",
        list(ops = TRUE, ns = TRUE,
-            args = TRUE, func = FALSE,
+            args = TRUE, dots = TRUE, func = FALSE,
             ipck = FALSE, S3 = TRUE, data = TRUE,
             help = TRUE, argdb = TRUE, fuzzy = FALSE,
             files = TRUE, # FIXME: deprecate in favour of quotes
             quotes = TRUE),
-       env = .CompletionEnv)
+       envir = .CompletionEnv)
 
 assign("options",
        list(package.suffix = "::",
             funarg.suffix = "=",
             function.suffix = "("),
-       env = .CompletionEnv)
+       envir = .CompletionEnv)
 
 ## These keeps track of attached packages and available help topics.
 ## Needs updating only when packages are attached.
-assign("attached_packages", character(0), env = .CompletionEnv)
-assign("help_topics", character(0), env = .CompletionEnv)
+assign("attached_packages", character(0), envir = .CompletionEnv)
+assign("help_topics", character(0), envir = .CompletionEnv)
 
 
 .FunArgEnv <- new.env(hash = TRUE, parent = emptyenv())
