@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2012   The R Core Team.
+ *  Copyright (C) 2012-2022   The R Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, a copy is available at
- *  http://www.r-project.org/Licenses/
+ *  https://www.R-project.org/Licenses/
  */
 
 #ifdef HAVE_CONFIG_H
@@ -26,6 +26,7 @@
 
 #include "utils.h"
 #include <R_ext/Rdynload.h>
+#include <R_ext/Visibility.h>
 
 
 #define CALLDEF(name, n)  {#name, (DL_FUNC) &name, n}
@@ -38,6 +39,8 @@ static const R_CallMethodDef CallEntries[] = {
     CALLDEF(objectSize, 1),
     CALLDEF(processevents, 0),
     CALLDEF(octsize, 1),
+    CALLDEF(charClass, 2),
+    CALLDEF(tzcode_type, 0),
 
     /* Sockets */
     CALLDEF(sockconnect, 2),
@@ -62,7 +65,6 @@ static const R_CallMethodDef CallEntries[] = {
     CALLDEF(getWindowsHandle, 1),
     CALLDEF(getWindowsHandles, 2),
     CALLDEF(loadRconsole, 1),
-    CALLDEF(memsize, 1),
     CALLDEF(shortpath, 1),
 #endif
 
@@ -73,17 +75,17 @@ static const R_CallMethodDef CallEntries[] = {
 
 static const R_ExternalMethodDef ExtEntries[] = {
 #ifdef Win32
-    EXTDEF(download, 6),
+    EXTDEF(download, 7),
 #else
-    EXTDEF(download, 5),
+    EXTDEF(download, 6),
 #endif
     EXTDEF(unzip, 7),
-    EXTDEF(Rprof, 8),
+    EXTDEF(Rprof, 9),
     EXTDEF(Rprofmem, 3),
 
     EXTDEF(countfields, 6),
     EXTDEF(readtablehead, 7),
-    EXTDEF(typeconvert, 5),
+    EXTDEF(typeconvert, 6),
     EXTDEF(writetable, 11),
 
     EXTDEF(addhistory, 1),
@@ -96,6 +98,17 @@ static const R_ExternalMethodDef ExtEntries[] = {
     EXTDEF(fileedit, 3),
     EXTDEF(selectlist, 4),
 
+    /* Hash Tables */
+    EXTDEF(hashtab_Ext, 2),
+    EXTDEF(gethash_Ext, 3),
+    EXTDEF(sethash_Ext, 3),
+    EXTDEF(remhash_Ext, 2),
+    EXTDEF(numhash_Ext, 1),
+    EXTDEF(typhash_Ext, 1),
+    EXTDEF(maphash_Ext, 2),
+    EXTDEF(clrhash_Ext, 1),
+    EXTDEF(ishashtab_Ext, 1),
+
 #ifdef Win32
     EXTDEF(winProgressBar, 6),
     EXTDEF(closeWinProgressBar, 1),
@@ -115,10 +128,7 @@ static const R_ExternalMethodDef ExtEntries[] = {
 };
 
 
-void
-#ifdef HAVE_VISIBILITY_ATTRIBUTE
-__attribute__ ((visibility ("default")))
-#endif
+void attribute_visible
 R_init_utils(DllInfo *dll)
 {
     R_registerRoutines(dll, NULL, CallEntries, NULL, ExtEntries);
