@@ -14,7 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, a copy is available at
- *  http://www.r-project.org/Licenses/
+ *  https://www.R-project.org/Licenses/
  *
  */
 /*
@@ -50,7 +50,7 @@
 #include <R_ext/Linpack.h>
 
 
-/*  These get the declarations of some routines refernced here but
+/*  These get the declarations of some routines referenced here but
     not explicitly declared.    This is necessary when we link with
     a C++ compiler because the linkage changes as the declarations
     are (currently) within extern "C" blocks.
@@ -89,7 +89,7 @@ static R_FortranMethodDef fortranMethods[] = {
 };
 
 
-void attribute_hidden
+attribute_hidden void
 R_init_base(DllInfo *dll)
 {
     R_registerRoutines(dll, NULL, callMethods, fortranMethods, NULL);
