@@ -1,14 +1,19 @@
 ### Testing the group methods  --- some also happens in ./Class+Meth.R
 
+## for R_DEFAULT_PACKAGES=NULL :
+library(stats)
+library(utils)
+
 library(Matrix)
 source(system.file("test-tools.R", package = "Matrix"))# identical3() etc
 assertErrV <- function(e) tools::assertError(e, verbose=TRUE)
 
 cat("doExtras:",doExtras,"\n")
+options(nwarnings = 1e4)
 
 set.seed(2001)
 
-mm <- Matrix(rnorm(50 * 7), nc = 7)
+mm <- Matrix(rnorm(50 * 7), ncol = 7)
 xpx <- crossprod(mm)# -> "factors" in mm !
 round(xpx, 3) # works via "Math2"
 
@@ -54,15 +59,13 @@ stopifnot(exprs = {
 
 ###--- sparse matrices ---------
 
-m <- Matrix(c(0,0,2:0), 3,5)
-(mC <- as(m, "dgCMatrix"))
+mC <- Matrix(c(0, 0, 2:0), 3, 5)
 sm <- sin(mC)
 stopifnot(class(sm) == class(mC), class(mC) == class(mC^2),
           dim(sm) == dim(mC),
           class(0 + 100*mC) == class(mC),
           all.equal(0.1 * ((0 + 100*mC)/10), mC),
           all.equal(sqrt(mC ^ 2), mC),
-          all.equal(m^m, mC^mC),
           identical(mC^2, mC * mC),
           identical(mC*2, mC + mC)
           )
@@ -72,7 +75,7 @@ x # sparse
 (x2 <- x + 10*t(x))
 stopifnot(is(x2, "sparseMatrix"),
           identical(x2, t(x*10 + t(x))),
-	  identical(x, as((x + 10) - 10, class(x))))
+	  identical(x, as((x + 10) - 10, "CsparseMatrix")))
 
 (px <- Matrix(x^x - 1))#-> sparse again
 stopifnot(px@i == c(3,4,1,4),
@@ -110,18 +113,20 @@ dsy <- lsy+1
 D3 <- Diagonal(x=4:2); L7 <- Diagonal(7) > 0
 validObject(xpp <- pack(round(xpx,2)))
 lsp <- xpp > 0
-(dsyU <- as(as(.diag2sT(D3), "dMatrix"), "denseMatrix"))
- lsyU <- as(as(.diag2sT(Diagonal(5) > 0), "lMatrix"), "denseMatrix")
+(dsyU <- .diag2dense(D3, "s"))
+ lsyU <- .diag2dense(Diagonal(5) > 0, "s")
 str(lsyU)
 stopifnot({
     isValid(dsyU,               "dsyMatrix") && dsyU@uplo == "U"
     isValid(dsyL <- t(dsyU),    "dsyMatrix") && dsyL@uplo == "L"
     isValid(dspU <- pack(dsyU), "dspMatrix") && dspU@uplo == "U"
     isValid(dspL <- pack(dsyL), "dspMatrix") && dspL@uplo == "L"
+    identical(dspU, t(dspL))
     isValid(lsyU,               "lsyMatrix") && lsyU@uplo == "U"
     isValid(lsyL <- t(lsyU),    "lsyMatrix") && lsyL@uplo == "L"
     isValid(lspU <- pack(lsyU), "lspMatrix") && lspU@uplo == "U"
     isValid(lspL <- pack(lsyL), "lspMatrix") && lspL@uplo == "L"
+    identical(lspL, t(lspU))
     ##
     ## log(x, <base>) -- was mostly *wrong* upto 2019-10 [Matrix <= 1.2-17]
     all.equal(log(abs(dsy), 2), log2(abs(dsy)))
@@ -140,6 +145,41 @@ stopifnot({
     all.equal(log(abs( D3), 2), log2(abs( D3)))
     all.equal(log(abs( L7), 2), log2(abs( L7)))
 })
+showProc.time()
+
+## is.finite() -- notably for symmetric packed / uplo="L" with NA :
+spU <- new("dspMatrix", Dim = c(3L, 3L), x = c(0, NA, 0, NA, NA, 0),           uplo = "U")
+sU  <- new("dsyMatrix", Dim = c(3L, 3L), x = c(1, NA, NA, NA, 1, NA, 8, 2, 1), uplo = "U")
+sL  <- t(sU)
+spL <- t(spU)
+trU <- triu(spU)
+trL <- tril(spL)
+stopifnot(exprs = {
+    spL@uplo == "L"
+    trU@uplo == "U"
+    trL@uplo == "L"
+    identical(trU, triu(spL))
+    identical(trL, tril(spU))
+})
+isU <- is.finite(sU)
+isL <- is.finite(sL)
+stopifnot(exprs = {
+    identical(isU, t(isL))
+    all(isU == isL)
+    which(!isU, arr.ind = TRUE) == c(2:1, 1:2)
+})
+isFu <- is.finite(spU)
+isFl <- is.finite(spL)
+isFtu <- is.finite(trU)
+isFtl <- is.finite(trL)
+stopifnot(exprs = {
+    all(isFu == diag(TRUE, 3))
+    all(isFu == isFl) # has failed till 2022-06-11
+    isTriangular(isFtu)
+    isTriangular(isFtl)
+    identical(rep(TRUE, 6), pack(tril(isFtu))@x)
+    identical(rep(TRUE, 6), pack(triu(isFtl))@x)
+})
 
 showProc.time()
 set.seed(111)
@@ -168,7 +208,9 @@ local({
                                grep("bogomips", readLines("/proc/cpuinfo"),
                                     ignore.case=TRUE, # e.g. ARM : "BogoMIPS"
                                     value=TRUE)[[1]])))
-        if(is.numeric(mips) && all(mips) > 0)
+        print(mips)
+        if(is.numeric(mips) && all(mips) > 0 && doExtras)
+                                        # doExtras: valgrind (2023-07-26) gave large 'st[1]'
         stopifnot(st[1] < 1000/mips)# ensure there was no gross inefficiency
     }
 })
@@ -195,7 +237,7 @@ nm1 <- as(lm1, "nMatrix")
 
 stopifnot(validObject(lm1), validObject(lm2),
           validObject(nm1), validObject(nm2),
-          identical(dsc, as(dsc * as(lm1, "dMatrix"), "dsCMatrix")))
+          identical(dsc, dsc * as(lm1, "dMatrix")))
 
 crossprod(lm1) # lm1: "lsC*"
 cnm1 <- crossprod(nm1)
@@ -232,8 +274,8 @@ stopifnot(identical(crossprod(lm1),# "lgC": here works!
 	  identical(lm1, lm1 | lm2))
 
 ddsc <- kronecker(Diagonal(7), dsc)
-isValid(ddv <- rowSums(ddsc, sparse=TRUE), "sparseVector")
-sv <- colSums(kC <- kronecker(mC,kronecker(mC,mC)), sparse=TRUE)
+isValid(ddv <- rowSums(ddsc, sparseResult=TRUE), "sparseVector")
+sv <- colSums(kC <- kronecker(mC,kronecker(mC,mC)), sparseResult=TRUE)
 EQ <- ddv == rowSums(ddsc)
 na.ddv <- is.na(ddv)
 sM <- Matrix(pmax(0, round(rnorm(50*15, -1.5), 2)), 50,15)
@@ -275,11 +317,11 @@ z[sample(77,15)] <- 0
 abs(D) >= 0.5       # logical sparse
 
 ## For the checks below, remove some and add a few more objects:
-rm(list= ls(pat="^.[mMC]?$"))
+rm(list= ls(pattern="^.[mMC]?$"))
 T3 <- Diagonal(3) > 0; stopifnot(T3@diag == "U") # "uni-diagonal"
 validObject(dtp <- pack(as(dt3, "denseMatrix")))
 stopifnot(exprs = {
-    isValid(lsC <- as(lsp, "sparseMatrix"), "lsCMatrix")
+    isValid(lsC <- as(lsp, "CsparseMatrix"), "lsCMatrix")
     ## 0-extent matrices {fixes in Feb.2019}:
     isValid(L00 <- L7[FALSE,FALSE], "ldiMatrix")
     isValid(x60 <- x2[,FALSE],      "dgCMatrix")
@@ -304,6 +346,32 @@ MatDims <- t(vapply(Mat.objs, function(nm) dim(get(nm)), 0:1))
 ## Nice summary info :
 noquote(cbind(Mcl[Mat.objs], format(MatDims)))
 
+## dtCMatrix, uplo="L" :
+(CtL <- t(as(Diagonal(x=4:2), "CsparseMatrix")))
+m2 <- cbind(c(0, NA, NA),
+            c(0,  0, NA), 0)
+op <- options(Matrix.verbose = 2)
+r <- CtL > m2 # failed in Matrix <= 1.4-1, with
+## Compare <Csparse> -- "dtCMatrix" > "dtCMatrix" :
+stopifnot(identical(is.na(m2), unname(as.matrix(is.na(r)))), diag(r), isDiagonal(triu(r)))
+M <- new("dtCMatrix", i = c(0L, 0:1, 0:2), p = c(0:1, 3L, 6L),
+         x = c(10,1, 10,1, 1,10), Dim = c(3L, 3L), uplo = "U")
+m2 <- matrix(c(0, NA, NA, 0, 0, NA, 0, 0, 0), 3)
+r <- M & m2 # failed in Matrix <= 1.4-1
+assert.EQ.mat(M        | m2 -> ro,
+              as.mat(M)| m2, tol=0)
+D4 <- Diagonal(x=0+ 4:2)
+rd <- D4 | m2 # gave  invalid class "ltTMatrix" object: uplo='U' must not have sparse entries below the diagonal
+M2 <- Matrix(m2); T2 <- Matrix:::.diag2T.smart(D4, M2, kind="l")
+stopifnot(exprs = {
+    all(!r)
+    ## fix in .do.Logic.lsparse() {needed uplo="L"}
+    identical(rd,    T2                   |    M2)
+    identical(rd, as(T2, "CsparseMatrix") | as(M2, "lMatrix"))
+})
+
+options(op)
+
 if(!doExtras && !interactive()) q("no") ## (saving testing time)
 
 ### Systematically look at all "Ops" group generics for "all" Matrix classes
@@ -367,7 +435,8 @@ for(gr in getGroupMembers("Ops")) {
               cat(sprintf("\n %s %s %s gave not identical r4 & R4:\n",
                           nm, f, oM));     print(r4); print(R4)
               C1 <- (eq <- R4 == r4) | (N4 <- as.logical((nr4 <- is.na(eq)) & !is.finite(R4)))
-              if(isTRUE(all(C1)))
+              if(isTRUE(all(C1)) || isTRUE(all.equal(as.mat(R4), r4,
+                                                     tolerance = 1e-14)))
                   cat(sprintf(
                       " --> %s %s %s (ok): only difference is %s (matrix) and %s (Matrix)\n",
                       M.knd(M), f, M.knd(M2)
@@ -418,7 +487,8 @@ assertErrV(m == mm)
 ## in R <= 3.3.x, relop returned logical(0) and  m + 2:3  returned numeric(0)
 ##
 ## arithmetic, logic, and comparison (relop) -- inconsistency for 1x1 array o <vector >= 2>:
-(m1 <- Matrix(1,1,1, dimnames=list("Ro","col")))
+## FIXME: desired errors are _not_ thrown for ddiMatrix (when doDiag=TRUE)
+(m1 <- Matrix(1, 1L, 1L, dimnames = list("Ro", "col"), doDiag = FALSE))
 ##    col
 ## Ro   1
 ## Before Sep.2016, here, Matrix was the *CONTRARY* to R:
@@ -460,7 +530,7 @@ for(f in c(mM, mM2)) {
     } else { ## (almost always:) matrix result
         assert.EQ.mat(R, r)
 	## check preservation of properties, notably super class
-	if(prod(dim(M)) > 1 && is(M, "diagonalMatrix"  ) && isDiagonal  (R) && !is(R, "diagonalMatrix"  )) doStop()
+	if(prod(dim(M)) > 1 && is(M, "diagonalMatrix"  ) && isDiagonal(R) && !is(R, "diagonalMatrix"  )) doStop()
 	if(prod(dim(M)) > 1 && is(M, "triangularMatrix") && (iT <- isTriangular(R)) && attr(iT, "kind") == M@uplo &&
            !is(R, "triangularMatrix")) doStop()
     }
@@ -472,8 +542,6 @@ showProc.time()
 ##
 cat("Checking the Summary group generics for a set of arguments:\n",
     "------------ ======= ------------------------------------------------\n", sep='')
-doStop <- function()
-    warning("**Summary: ", f,"(<",class(M),">) is not all.equal(..)", immediate.=TRUE)
 for(f in getGroupMembers("Summary")) {
   cat(sprintf("%-9s :\n %-7s\n", paste0('"',f,'"'), paste(rep("-", nchar(f)), collapse="")))
   givesVec <- f %in% mVec
@@ -492,6 +560,16 @@ for(f in getGroupMembers("Summary")) {
   if(length(warnings())) print(summary(warnings()))
 }
 
-
+## <Math>(x) behaved incorrectly in Matrix <= 1.4-1
+## for unit diagonal 'x' when f(0) == 0 and f(1) != 1
+Dn <- list(c("a", "b"), c("A", "B"))
+udi <- new("ddiMatrix", Dim = c(2L, 2L), Dimnames = Dn, diag = "U")
+utC <- new("dtCMatrix", Dim = c(2L, 2L), Dimnames = Dn, diag = "U",
+           p = integer(3L))
+utr <- new("dtrMatrix", Dim = c(2L, 2L), Dimnames = Dn, diag = "U",
+           x = double(4L))
+sinu <- `dimnames<-`(sin(diag(2L)), Dn)
+for(u in list(udi, utC, utr))
+    stopifnot(identical(as(sin(u), "matrix"), sinu))
 
 cat('Time elapsed: ', proc.time(),'\n') # for ``statistical reasons''
